# Project Marvin

[![Quality Gate Status](https://sonar.us-east.devhub-cloud.cisco.com/api/project_badges/measure?project=panoptica-marvin&metric=alert_status&token=sqb_aa88cc46acaf5b3d2d9f8c09843024776e7a66df)](https://sonar.us-east.devhub-cloud.cisco.com/dashboard?id=panoptica-marvin)
[![Coverage](https://sonar.us-east.devhub-cloud.cisco.com/api/project_badges/measure?project=panoptica-marvin&metric=coverage&token=sqb_aa88cc46acaf5b3d2d9f8c09843024776e7a66df)](https://sonar.us-east.devhub-cloud.cisco.com/dashboard?id=panoptica-marvin)
[![Code Smells](https://sonar.us-east.devhub-cloud.cisco.com/api/project_badges/measure?project=panoptica-marvin&metric=code_smells&token=sqb_aa88cc46acaf5b3d2d9f8c09843024776e7a66df)](https://sonar.us-east.devhub-cloud.cisco.com/dashboard?id=panoptica-marvin)
[![Vulnerabilities](https://sonar.us-east.devhub-cloud.cisco.com/api/project_badges/measure?project=panoptica-marvin&metric=vulnerabilities&token=sqb_aa88cc46acaf5b3d2d9f8c09843024776e7a66df)](https://sonar.us-east.devhub-cloud.cisco.com/dashboard?id=panoptica-marvin)


Marvin is intended to provide protection for LLM-driven components of software systems.

We do so by inspecting prompts (and in the future LLM responses to these prompts) with NLP techniques to determine if 
they are suspected to malicious activity. 

## Components
* [Marvin SDK](./sdk) - Python SDK to programmatically integrate system with LLM protection
* [Prompt Inspection Server](./prompt_inspection_server) - HTTP server to serve the requests made by Marvin SDK
* [Demo Chat App](./test_app) - Simple streamlit-based chat app to demo Marvin capabilities

## Code Generation
### Python
We use auto-generated request/response models from OpenAPI spec,
using [datamodel-code-generator](https://koxudaxi.github.io/datamodel-code-generator/). 

The server spec is under [prompt/openapi/openapi.yaml](openapi/prompt_inspection.yaml).
We use its generated artifacts in both the client and the server.

_Note: Currently we only generate model files and not the client/server code utilizing them._

#### Installing `datamodel-code-generator`
```shell
pip install datamodel-code-generator
```
### Go
#### Installing `oapi-codegen`
```shell
go install github.com/oapi-codegen/oapi-codegen/v2/cmd/oapi-codegen@latest
```

### Run UnitTests
```shell
make test
```

### Run Helm Chart Test and Linting
```shell
make helm-test
```

### Generating the `models.py` 
The following snippet would generate the `models.py` file with all the schema objects declared in `openapi.yaml`, 
and copy the generated file to its location in the SDK folder.

```shell
make generate
```

## CODEOWNERS
Please modify this file to include your team and CODEOWNER rules

## Jenkins pipeline
Please refer to the [SRE-Pipeline-Library](https://wwwin-github.cisco.com/pages/eti/sre-pipeline-library/) documentation for further information on how to modify your Jenkinsfile to your needs

### `docker-compose.yaml`
This file will create a system that includes postgres and the backend.
to start follow this steps
```shell
docker-compose -f docker-compose.yaml up db --build -d
```
ssh into the db and create the forensic schema
```shell
create database marvin;
\c marvin
create schema "attack-analytics";
create schema "forensic";
create schema "auth";
```
```shell
docker-compose -f docker-compose.yaml up db --build -d
docker-compose -f docker-compose.yaml up bp-migration --build -d
docker-compose -f docker-compose.yaml up auth-migration --build -d
docker-compose -f docker-compose.yaml up forensic-migration --build -d
docker-compose -f docker-compose.yaml up forensic --build -d
```

### system tests
If you want to check a test you just added, follow this steps:
1. Let's say you need to add test_flow_name = 'send_complex_prompt', and want to test category = 'dan' 
2. go to `.github/workflows/run-system-tests.yaml` and add test_flow_name choice: 'send_complex_prompt', add attack_category choice: 'dan'
3. go to `.github/workflows/main.yaml` and add to run-system-tests strategy matrix under xss_injection. E.g.,
```yaml
        include:
          - test_flow_name: send_prompt
            attack_category: xss_injection
          - test_flow_name: send_complex_prompt
            attack_category: dan 
```

## Python requirements
we use poetry CLI to handle our python packages.
prerequisites: `brew install poetry`
to run poetry cd to a folder containing `pyproject.toml` file. example of poetry handling:
1. poetry install --no-root - install current dependencies into your venv
2. poetry update - update and install latest packages with given constraints from pyproject.toml
3. poetry lock - update and checks dependencies in lock file

## Metrics - Prometheus
we are using Prometheus to handle push metrics to grafana. 
To test the API schemes being pushed you can test it locally or by port-forwarding.
The grafana dashboard is handled in [here](https://wwwin-github.cisco.com/eti/sre-helm-charts/tree/main/grafana-dashboards-organization/dashboards-json) in marvin-backend for all clusters
in grafana simply replace the cluster name to that of the dev/ stage/ production environments
#### Locally
here we are trying to test auth-migration metrics schema:
```shell
docker-compose -f docker-compose.yaml up db --build pushgateway
docker-compose -f docker-compose.yaml up db --build auth-migration
```
in your browser go to http://localhost:9091/ and check the schema (you should see job=db_migration_auth).

#### port-forwarding
example of port-forwarding for dev:
```shell
kubectl -n prometheus-pushgateway port-forward svc/marvin-dev-use2-1-prometheus-pushgateway 9091:9091
```
then, check the push metrics to gateway schema in http://localhost:9091/